@@ -1,5 +1,5 @@
 use crypto_ws_client::{DeribitWSClient, WSClient};
-use std::sync::{Arc, Mutex};
+use std::sync::mpsc::{Receiver, Sender};
 
 #[macro_use]
 mod utils;
@@ -20,7 +20,7 @@ fn deribit_all_trades() {
 #[cfg(test)]
 mod deribit_inverse_future {
     use crypto_ws_client::{DeribitWSClient, WSClient};
-    use std::sync::{Arc, Mutex};
+    use std::sync::mpsc::{Receiver, Sender};
 
     #[test]
     fn subscribe() {
@@ -77,7 +77,7 @@ mod deribit_inverse_future {
 #[cfg(test)]
 mod deribit_inverse_swap {
     use crypto_ws_client::{DeribitWSClient, WSClient};
-    use std::sync::{Arc, Mutex};
+    use std::sync::mpsc::{Receiver, Sender};
 
     #[test]
     fn subscribe() {
@@ -137,7 +137,7 @@ mod deribit_inverse_swap {
 #[cfg(test)]
 mod deribit_option {
     use crypto_ws_client::{DeribitWSClient, WSClient};
-    use std::sync::{Arc, Mutex};
+    use std::sync::mpsc::{Receiver, Sender};
 
     const SYMBOLS: &'static [&str] = &[
         "BTC-26SEP21-40000-P",
